@@ -1,10 +1,10 @@
 /*
- * Copyright (C) 2010-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2010-2020 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #include "config.h"
@@ -234,14 +234,14 @@ static int _init_buffers(knet_handle_t knet_h)
 	}
 	memset(knet_h->pingbuf, 0, KNET_HEADER_PING_SIZE);
 
-	knet_h->pmtudbuf = malloc(KNET_PMTUD_SIZE_V6);
+	knet_h->pmtudbuf = malloc(KNET_PMTUD_SIZE_V6 + KNET_HEADER_ALL_SIZE);
 	if (!knet_h->pmtudbuf) {
 		savederrno = errno;
 		log_err(knet_h, KNET_SUB_HANDLE, "Unable to allocate memory for pmtud buffer: %s",
 			strerror(savederrno));
 		goto exit_fail;
 	}
-	memset(knet_h->pmtudbuf, 0, KNET_PMTUD_SIZE_V6);
+	memset(knet_h->pmtudbuf, 0, KNET_PMTUD_SIZE_V6 + KNET_HEADER_ALL_SIZE);
 
 	for (i = 0; i < PCKT_FRAG_MAX; i++) {
 		bufsize = ceil((float)KNET_MAX_PACKET_SIZE / (i + 1)) + KNET_HEADER_ALL_SIZE + KNET_DATABUFSIZE_CRYPT_PAD;
@@ -309,7 +309,10 @@ static int _init_buffers(knet_handle_t knet_h)
 	}
 	memset(knet_h->send_to_links_buf_compress, 0, KNET_DATABUFSIZE_COMPRESS);
 
-	memset(knet_h->knet_transport_fd_tracker, KNET_MAX_TRANSPORTS, sizeof(knet_h->knet_transport_fd_tracker));
+	memset(knet_h->knet_transport_fd_tracker, 0, sizeof(knet_h->knet_transport_fd_tracker));
+	for (i = 0; i < KNET_MAX_FDS; i++) {
+		knet_h->knet_transport_fd_tracker[i].transport = KNET_MAX_TRANSPORTS;
+	}
 
 	return 0;
 
@@ -453,9 +456,24 @@ static void _close_epolls(knet_handle_t knet_h)
 static int _start_threads(knet_handle_t knet_h)
 {
 	int savederrno = 0;
+	pthread_attr_t attr;
 
 	set_thread_status(knet_h, KNET_THREAD_PMTUD, KNET_THREAD_REGISTERED);
-	savederrno = pthread_create(&knet_h->pmtud_link_handler_thread, 0,
+
+	savederrno = pthread_attr_init(&attr);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_HANDLE, "Unable to init pthread attributes: %s",
+			strerror(savederrno));
+		goto exit_fail;
+	}
+	savederrno = pthread_attr_setstacksize(&attr, KNET_THREAD_STACK_SIZE);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_HANDLE, "Unable to set stack size attribute: %s",
+			strerror(savederrno));
+		goto exit_fail;
+	}
+
+	savederrno = pthread_create(&knet_h->pmtud_link_handler_thread, &attr,
 				    _handle_pmtud_link_thread, (void *) knet_h);
 	if (savederrno) {
 		log_err(knet_h, KNET_SUB_HANDLE, "Unable to start pmtud link thread: %s",
@@ -464,7 +482,7 @@ static int _start_threads(knet_handle_t knet_h)
 	}
 
 	set_thread_status(knet_h, KNET_THREAD_DST_LINK, KNET_THREAD_REGISTERED);
-	savederrno = pthread_create(&knet_h->dst_link_handler_thread, 0,
+	savederrno = pthread_create(&knet_h->dst_link_handler_thread, &attr,
 				    _handle_dst_link_handler_thread, (void *) knet_h);
 	if (savederrno) {
 		log_err(knet_h, KNET_SUB_HANDLE, "Unable to start dst cache thread: %s",
@@ -473,7 +491,7 @@ static int _start_threads(knet_handle_t knet_h)
 	}
 
 	set_thread_status(knet_h, KNET_THREAD_TX, KNET_THREAD_REGISTERED);
-	savederrno = pthread_create(&knet_h->send_to_links_thread, 0,
+	savederrno = pthread_create(&knet_h->send_to_links_thread, &attr,
 				    _handle_send_to_links_thread, (void *) knet_h);
 	if (savederrno) {
 		log_err(knet_h, KNET_SUB_HANDLE, "Unable to start datafd to link thread: %s",
@@ -482,7 +500,7 @@ static int _start_threads(knet_handle_t knet_h)
 	}
 
 	set_thread_status(knet_h, KNET_THREAD_RX, KNET_THREAD_REGISTERED);
-	savederrno = pthread_create(&knet_h->recv_from_links_thread, 0,
+	savederrno = pthread_create(&knet_h->recv_from_links_thread, &attr,
 				    _handle_recv_from_links_thread, (void *) knet_h);
 	if (savederrno) {
 		log_err(knet_h, KNET_SUB_HANDLE, "Unable to start link to datafd thread: %s",
@@ -491,7 +509,7 @@ static int _start_threads(knet_handle_t knet_h)
 	}
 
 	set_thread_status(knet_h, KNET_THREAD_HB, KNET_THREAD_REGISTERED);
-	savederrno = pthread_create(&knet_h->heartbt_thread, 0,
+	savederrno = pthread_create(&knet_h->heartbt_thread, &attr,
 				    _handle_heartbt_thread, (void *) knet_h);
 	if (savederrno) {
 		log_err(knet_h, KNET_SUB_HANDLE, "Unable to start heartbeat thread: %s",
@@ -499,6 +517,15 @@ static int _start_threads(knet_handle_t knet_h)
 		goto exit_fail;
 	}
 
+	savederrno = pthread_attr_destroy(&attr);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_HANDLE, "Unable to destroy pthread attributes: %s",
+			strerror(savederrno));
+		/*
+		 * Do not return error code. Error is not critical.
+		 */
+	}
+
 	return 0;
 
 exit_fail:
@@ -635,9 +662,10 @@ knet_handle_t knet_handle_new_ex(knet_node_id_t host_id,
 
 	if (_init_shlib_tracker(knet_h) < 0) {
 		savederrno = errno;
-		log_err(knet_h, KNET_SUB_HANDLE, "Unable to init handles traceker: %s",
+		log_err(knet_h, KNET_SUB_HANDLE, "Unable to init handle tracker: %s",
 			strerror(savederrno));
 		errno = savederrno;
+		pthread_mutex_unlock(&handle_config_mutex);
 		goto exit_fail;
 	}
 
@@ -782,7 +810,7 @@ int knet_handle_enable_sock_notify(knet_handle_t knet_h,
 						int error,
 						int errorno))
 {
-	int savederrno = 0, err = 0;
+	int savederrno = 0;
 
 	if (!knet_h) {
 		errno = EINVAL;
@@ -808,8 +836,7 @@ int knet_handle_enable_sock_notify(knet_handle_t knet_h,
 
 	pthread_rwlock_unlock(&knet_h->global_rwlock);
 
-	errno = err ? savederrno : 0;
-	return err;
+	return 0;
 }
 
 int knet_handle_add_datafd(knet_handle_t knet_h, int *datafd, int8_t *channel)
@@ -1172,12 +1199,89 @@ int knet_handle_setfwd(knet_handle_t knet_h, unsigned int enabled)
 		return -1;
 	}
 
-	knet_h->enabled = enabled;
-
 	if (enabled) {
+		knet_h->enabled = enabled;
 		log_debug(knet_h, KNET_SUB_HANDLE, "Data forwarding is enabled");
 	} else {
+		/*
+		 * notify TX and RX threads to flush the queues
+		 */
+		if (set_thread_flush_queue(knet_h, KNET_THREAD_TX, KNET_THREAD_QUEUE_FLUSH) < 0) {
+			log_debug(knet_h, KNET_SUB_HANDLE, "Unable to request queue flushing for TX thread");
+		}
+		if (set_thread_flush_queue(knet_h, KNET_THREAD_RX, KNET_THREAD_QUEUE_FLUSH) < 0) {
+			log_debug(knet_h, KNET_SUB_HANDLE, "Unable to request queue flushing for RX thread");
+		}
+	}
+
+	pthread_rwlock_unlock(&knet_h->global_rwlock);
+
+	/*
+	 * when disabling data forward, we need to give time to TX and RX
+	 * to flush the queues.
+	 *
+	 * the TX thread is the main leader here. When there is no more
+	 * data in the TX queue, we will also close traffic for RX.
+	 */
+	if (!enabled) {
+		/*
+		 * this usleep might be unnecessary, but wait_all_threads_flush_queue
+		 * adds extra locking delay.
+		 *
+		 * allow all threads to run free without extra locking interference
+		 * and then we switch to a more active wait in case the scheduler
+		 * has decided to delay one thread or another
+		 */
+		usleep(KNET_THREADS_TIMERES * 2);
+		wait_all_threads_flush_queue(knet_h);
+
+		/*
+		 * all threads have done flushing the queue, we can stop data forwarding
+		 */
+		savederrno = get_global_wrlock(knet_h);
+		if (savederrno) {
+			log_err(knet_h, KNET_SUB_HANDLE, "Unable to get write lock: %s",
+				strerror(savederrno));
+			errno = savederrno;
+			return -1;
+		}
+		knet_h->enabled = enabled;
 		log_debug(knet_h, KNET_SUB_HANDLE, "Data forwarding is disabled");
+		pthread_rwlock_unlock(&knet_h->global_rwlock);
+	}
+
+	errno = 0;
+	return 0;
+}
+
+int knet_handle_enable_access_lists(knet_handle_t knet_h, unsigned int enabled)
+{
+	int savederrno = 0;
+
+	if (!knet_h) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (enabled > 1) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	savederrno = get_global_wrlock(knet_h);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_HANDLE, "Unable to get write lock: %s",
+			strerror(savederrno));
+		errno = savederrno;
+		return -1;
+	}
+
+	knet_h->use_access_lists = enabled;
+
+	if (enabled) {
+		log_debug(knet_h, KNET_SUB_HANDLE, "Links access lists are enabled");
+	} else {
+		log_debug(knet_h, KNET_SUB_HANDLE, "Links access lists are disabled");
 	}
 
 	pthread_rwlock_unlock(&knet_h->global_rwlock);
@@ -1282,6 +1386,41 @@ int knet_handle_enable_pmtud_notify(knet_handle_t knet_h,
 	return 0;
 }
 
+int knet_handle_pmtud_set(knet_handle_t knet_h,
+			  unsigned int iface_mtu)
+{
+	int savederrno = 0;
+
+	if (!knet_h) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (iface_mtu > KNET_PMTUD_SIZE_V4) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	savederrno = pthread_rwlock_rdlock(&knet_h->global_rwlock);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_PMTUD, "Unable to get read lock: %s",
+			strerror(savederrno));
+		errno = savederrno;
+		return -1;
+	}
+
+	log_info(knet_h, KNET_SUB_PMTUD, "MTU manually set to: %u", iface_mtu);
+
+	knet_h->manual_mtu = iface_mtu;
+
+	force_pmtud_run(knet_h, KNET_SUB_PMTUD, 0);
+
+	pthread_rwlock_unlock(&knet_h->global_rwlock);
+
+	errno = 0;
+	return 0;
+}
+
 int knet_handle_pmtud_get(knet_handle_t knet_h,
 			  unsigned int *data_mtu)
 {
@@ -1336,11 +1475,10 @@ int knet_handle_crypto(knet_handle_t knet_h, struct knet_handle_crypto_cfg *knet
 		return -1;
 	}
 
-	crypto_fini(knet_h);
-
 	if ((!strncmp("none", knet_handle_crypto_cfg->crypto_model, 4)) || 
 	    ((!strncmp("none", knet_handle_crypto_cfg->crypto_cipher_type, 4)) &&
 	     (!strncmp("none", knet_handle_crypto_cfg->crypto_hash_type, 4)))) {
+		crypto_fini(knet_h);
 		log_debug(knet_h, KNET_SUB_CRYPTO, "crypto is not enabled");
 		err = 0;
 		goto exit_unlock;
@@ -1370,6 +1508,9 @@ int knet_handle_crypto(knet_handle_t knet_h, struct knet_handle_crypto_cfg *knet
 	}
 
 exit_unlock:
+	if (!err) {
+		force_pmtud_run(knet_h, KNET_SUB_CRYPTO, 1);
+	}
 	pthread_rwlock_unlock(&knet_h->global_rwlock);
 	errno = err ? savederrno : 0;
 	return err;
@@ -1537,7 +1678,6 @@ out_unlock:
 int knet_handle_get_stats(knet_handle_t knet_h, struct knet_handle_stats *stats, size_t struct_size)
 {
 	int savederrno = 0;
-	int err = 0;
 
 	if (!knet_h) {
 		errno = EINVAL;
@@ -1577,14 +1717,12 @@ int knet_handle_get_stats(knet_handle_t knet_h, struct knet_handle_stats *stats,
 	stats->size = sizeof(struct knet_handle_stats);
 
 	pthread_rwlock_unlock(&knet_h->global_rwlock);
-	errno = err ? savederrno : 0;
-	return err;
+	return 0;
 }
 
 int knet_handle_clear_stats(knet_handle_t knet_h, int clear_option)
 {
 	int savederrno = 0;
-	int err = 0;
 
 	if (!knet_h) {
 		errno = EINVAL;
@@ -1612,7 +1750,5 @@ int knet_handle_clear_stats(knet_handle_t knet_h, int clear_option)
 	}
 
 	pthread_rwlock_unlock(&knet_h->global_rwlock);
-	errno = err ? savederrno : 0;
-	return err;
+	return 0;
 }
-
