@@ -1,10 +1,10 @@
 /*
- * Copyright (C) 2010-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2010-2020 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #ifndef __LIBKNET_H__
@@ -19,7 +19,7 @@
 /**
  * @file libknet.h
  * @brief kronosnet API include file
- * @copyright Copyright (C) 2010-2019 Red Hat, Inc.  All rights reserved.
+ * @copyright Copyright (C) 2010-2020 Red Hat, Inc.  All rights reserved.
  *
  * Kronosnet is an advanced VPN system for High Availability applications.
  */
@@ -151,6 +151,7 @@ knet_handle_t knet_handle_new(knet_node_id_t host_id,
 
 /**
  * knet_handle_free
+ *
  * @brief Destroy a knet handle, free all resources
  *
  * knet_h   - pointer to knet_handle_t
@@ -165,6 +166,7 @@ int knet_handle_free(knet_handle_t knet_h);
 
 /**
  * knet_handle_enable_sock_notify
+ *
  * @brief Register a callback to receive socket events
  *
  * knet_h   - pointer to knet_handle_t
@@ -275,6 +277,7 @@ int knet_handle_add_datafd(knet_handle_t knet_h, int *datafd, int8_t *channel);
 
 /**
  * knet_handle_remove_datafd
+ *
  * @brief Remove a file descriptor from knet
  *
  * knet_h   - pointer to knet_handle_t
@@ -293,6 +296,7 @@ int knet_handle_remove_datafd(knet_handle_t knet_h, int datafd);
 
 /**
  * knet_handle_get_channel
+ *
  * @brief Get the channel associated with a file descriptor
  *
  * knet_h  - pointer to knet_handle_t
@@ -313,6 +317,7 @@ int knet_handle_get_channel(knet_handle_t knet_h, const int datafd, int8_t *chan
 
 /**
  * knet_handle_get_datafd
+ *
  * @brief Get the file descriptor associated with a channel
  *
  * knet_h   - pointer to knet_handle_t
@@ -333,6 +338,7 @@ int knet_handle_get_datafd(knet_handle_t knet_h, const int8_t channel, int *data
 
 /**
  * knet_recv
+ *
  * @brief Receive data from knet nodes
  *
  * knet_h   - pointer to knet_handle_t
@@ -355,6 +361,7 @@ ssize_t knet_recv(knet_handle_t knet_h,
 
 /**
  * knet_send
+ *
  * @brief Send data to knet nodes
  *
  * knet_h   - pointer to knet_handle_t
@@ -495,6 +502,45 @@ int knet_handle_enable_filter(knet_handle_t knet_h,
 
 int knet_handle_setfwd(knet_handle_t knet_h, unsigned int enabled);
 
+/**
+ * knet_handle_enable_access_lists
+ *
+ * @brief Enable or disable usage of access lists (default: off)
+ *
+ * knet_h   - pointer to knet_handle_t
+ *
+ * enable   - set to 1 to use access lists, 0 to disable access_lists.
+ *
+ * @return
+ * knet_handle_enable_access_lists returns
+ * 0 on success
+ * -1 on error and errno is set.
+ *
+ * access lists are bound to links. There are 2 types of links:
+ * 1) point to point, where both source and destinations are well known
+ *    at configuration time.
+ * 2) open links, where only the source is known at configuration time.
+ *
+ * knet will automatically generate access lists for point to point links.
+ *
+ * For open links, knet provides 4 API calls to manipulate access lists:
+ * knet_link_add_acl(3), knet_link_rm_acl(3), knet_link_insert_acl(3)
+ * and knet_link_clear_acl(3).
+ * Those API calls will work exclusively on open links as they
+ * are of no use on point to point links.
+ *
+ * knet will not enforce any access list unless specifically enabled by
+ * knet_handle_enable_access_lists(3).
+ *
+ * From a security / programming perspective we recommend:
+ * - create the knet handle
+ * - enable access lists
+ * - configure hosts and links
+ * - configure access lists for open links
+ */
+
+int knet_handle_enable_access_lists(knet_handle_t knet_h, unsigned int enabled);
+
 #define KNET_PMTUD_DEFAULT_INTERVAL 60
 
 /**
@@ -568,6 +614,32 @@ int knet_handle_enable_pmtud_notify(knet_handle_t knet_h,
 						void *private_data,
 						unsigned int data_mtu));
 
+/**
+ * knet_handle_pmtud_set
+ *
+ * @brief Set the current interface MTU
+ *
+ * knet_h    - pointer to knet_handle_t
+ *
+ * iface_mtu - current interface MTU, value 0 to 65535. 0 will
+ *             re-enable automatic MTU discovery.
+ *             In a setup with multiple interfaces, please specify
+ *             the lowest MTU between the selected intefaces.
+ *             knet will automatically adjust this value for
+ *             all headers overhead and set the correct data_mtu.
+ *             data_mtu can be retrivied with knet_handle_pmtud_get(3)
+ *             or applications will receive a pmtud_nofity event
+ *             if enabled via knet_handle_enable_pmtud_notify(3).
+ *
+ * @return
+ * knet_handle_pmtud_set returns
+ * 0 on success
+ * -1 on error and errno is set.
+ */
+
+int knet_handle_pmtud_set(knet_handle_t knet_h,
+			  unsigned int iface_mtu);
+
 /**
  * knet_handle_pmtud_get
  *
@@ -587,8 +659,7 @@ int knet_handle_pmtud_get(knet_handle_t knet_h,
 				unsigned int *data_mtu);
 
 
-
-#define KNET_MIN_KEY_LEN  256
+#define KNET_MIN_KEY_LEN  128
 #define KNET_MAX_KEY_LEN 4096
 
 struct knet_handle_crypto_cfg {
@@ -618,7 +689,7 @@ struct knet_handle_crypto_cfg {
  *                         It can be set to "none" to disable
  *                         encryption.
  *                         Currently supported by "nss" model:
- *                         "3des", "aes128", "aes192" and "aes256".
+ *                         "aes128", "aes192" and "aes256".
  *                         "openssl" model supports more modes and it strictly
  *                         depends on the openssl build. See: EVP_get_cipherbyname
  *                         openssl API call for details.
@@ -655,9 +726,7 @@ struct knet_handle_crypto_cfg {
  *              1) failure to obtain locking
  *              2) errors to initializing the crypto level.
  *   This can happen even in subsequent calls to knet_handle_crypto.
- *   A failure in crypto init, might leave your traffic unencrypted!
- *   It's best to stop data forwarding (see knet_handle_setfwd(3)), change crypto config,
- *   start forward again.
+ *   A failure in crypto init will restore the previous crypto configuration.
  *
  * @return
  * knet_handle_crypto returns:
@@ -1021,15 +1090,15 @@ int knet_host_get_host_list(knet_handle_t knet_h,
 /**
  * knet_host_set_policy
  *
- * knet_h   - pointer to knet_handle_t
- *
  * @brief Set the switching policy for a host's links
  *
+ * knet_h   - pointer to knet_handle_t
+ *
  * host_id  - see knet_host_add(3)
  *
  * policy   - there are currently 3 kind of simple switching policies
  *            based on link configuration.
- *            KNET_LINK_POLICY_PASSIVE - the active link with the lowest
+ *            KNET_LINK_POLICY_PASSIVE - the active link with the highest
  *                                       priority will be used.
  *                                       if one or more active links share
  *                                       the same priority, the one with
@@ -1138,7 +1207,7 @@ struct knet_host_status {
 };
 
 /**
- * knet_host_status_get
+ * knet_host_get_status
  *
  * @brief Get the status of a host
  *
@@ -1433,6 +1502,166 @@ int knet_link_get_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 
 int knet_link_clear_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id);
 
+/*
+ * Access lists management for open links
+ * see also knet_handle_enable_access_lists(3)
+ */
+
+/**
+ * check_type_t
+ * @brief address type enum for knet access lists
+ *
+ * CHECK_TYPE_ADDRESS is the equivalent of a single entry / IP address.
+ *                    for example: 10.1.9.3
+ *                    and the entry is stored in ss1. ss2 can be NULL.
+ *
+ * CHECK_TYPE_MASK    is used to configure network/netmask.
+ *                    for example: 192.168.0.0/24
+ *                    the network is stored in ss1 and the netmask in ss2.
+ *
+ * CHECK_TYPE_RANGE   defines a value / range of ip addresses.
+ *                    for example: 172.16.0.1-172.16.0.10
+ *                    the start is stored in ss1 and the end in ss2.
+ *
+ * Please be aware that the above examples refer only to IP based protocols.
+ * Other protocols might use ss1 and ss2 in slightly different ways.
+ * At the moment knet only supports IP based protocol, though that might change
+ * in the future.
+ */
+
+typedef enum {
+	CHECK_TYPE_ADDRESS,
+	CHECK_TYPE_MASK,
+	CHECK_TYPE_RANGE
+} check_type_t;
+
+/**
+ * check_acceptreject_t
+ *
+ * @brief enum for accept/reject in knet access lists
+ *
+ * accept or reject incoming packets defined in the access list entry
+ */
+
+typedef enum {
+	CHECK_ACCEPT,
+	CHECK_REJECT
+} check_acceptreject_t;
+
+/**
+ * knet_link_add_acl
+ *
+ * @brief Add access list entry to an open link
+ *
+ * knet_h    - pointer to knet_handle_t
+ *
+ * host_id   - see knet_host_add(3)
+ *
+ * link_id   - see knet_link_set_config(3)
+ *
+ * ss1 / ss2 / type / acceptreject - see typedef definitions for details
+ *
+ * IMPORTANT: the order in which access lists are added is critical and it
+ *            is left to the user to add them in the right order. knet
+ *            will not attempt to logically sort them.
+ *
+ *            For example:
+ *            1 - accept from 10.0.0.0/8
+ *            2 - reject from 10.0.0.1/32
+ *
+ *            is not the same as:
+ *
+ *            1 - reject from 10.0.0.1/32
+ *            2 - accept from 10.0.0.0/8
+ *
+ *            In the first example, rule number 2 will never match because
+ *            packets from 10.0.0.1 will be accepted by rule number 1.
+ *
+ * @return
+ * knet_link_add_acl returns
+ * 0 on success.
+ * -1 on error and errno is set.
+ */
+
+int knet_link_add_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
+		      struct sockaddr_storage *ss1,
+		      struct sockaddr_storage *ss2,
+		      check_type_t type, check_acceptreject_t acceptreject);
+
+/**
+ * knet_link_insert_acl
+ *
+ * @brief Insert access list entry to an open link at given index
+ *
+ * knet_h    - pointer to knet_handle_t
+ *
+ * host_id   - see knet_host_add(3)
+ *
+ * link_id   - see knet_link_set_config(3)
+ *
+ * index     - insert at position "index" where 0 is the first entry and -1
+ *             appends to the current list.
+ *
+ * ss1 / ss2 / type / acceptreject - see typedef definitions for details
+ *
+ * @return
+ * knet_link_insert_acl returns
+ * 0 on success.
+ * -1 on error and errno is set.
+ */
+
+int knet_link_insert_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
+			 int index,
+			 struct sockaddr_storage *ss1,
+			 struct sockaddr_storage *ss2,
+			 check_type_t type, check_acceptreject_t acceptreject);
+
+/**
+ * knet_link_rm_acl
+ *
+ * @brief Remove access list entry from an open link
+ *
+ * knet_h    - pointer to knet_handle_t
+ *
+ * host_id   - see knet_host_add(3)
+ *
+ * link_id   - see knet_link_set_config(3)
+ *
+ * ss1 / ss2 / type / acceptreject - see typedef definitions for details
+ *
+ * IMPORTANT: the data passed to this API call must match exactly that passed
+ *            to knet_link_add_acl(3).
+ *
+ * @return
+ * knet_link_rm_acl returns
+ * 0 on success.
+ * -1 on error and errno is set.
+ */
+
+int knet_link_rm_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
+		     struct sockaddr_storage *ss1,
+		     struct sockaddr_storage *ss2,
+		     check_type_t type, check_acceptreject_t acceptreject);
+
+/**
+ * knet_link_clear_acl
+ *
+ * @brief Remove all access list entries from an open link
+ *
+ * knet_h    - pointer to knet_handle_t
+ *
+ * host_id   - see knet_host_add(3)
+ *
+ * link_id   - see knet_link_set_config(3)
+ *
+ * @return
+ * knet_link_clear_acl returns
+ * 0 on success.
+ * -1 on error and errno is set.
+ */
+
+int knet_link_clear_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id);
+
 /**
  * knet_link_set_enable
  *
@@ -1849,6 +2078,7 @@ int knet_link_get_status(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 #define KNET_SUB_LZO2COMP      73 /* compress_lzo.c */
 #define KNET_SUB_LZMACOMP      74 /* compress_lzma.c */
 #define KNET_SUB_BZIP2COMP     75 /* compress_bzip2.c */
+#define KNET_SUB_ZSTDCOMP      76 /* compress_zstd.c */
 
 #define KNET_SUB_UNKNOWN       UINT8_MAX - 1
 #define KNET_MAX_SUBSYSTEMS    UINT8_MAX
@@ -1933,7 +2163,7 @@ struct knet_log_msg {
 };
 
 /**
- * knet_log_set_log_level
+ * knet_log_set_loglevel
  *
  * @brief Set the logging level for a subsystem
  *
@@ -1956,7 +2186,7 @@ int knet_log_set_loglevel(knet_handle_t knet_h, uint8_t subsystem,
 			  uint8_t level);
 
 /**
- * knet_log_get_log_level
+ * knet_log_get_loglevel
  *
  * @brief Get the logging level for a subsystem
  *
