@@ -1,9 +1,9 @@
 /*
- * Copyright (C) 2016-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2020 Red Hat, Inc.  All rights reserved.
  *
  * Author: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under GPL-2.0+
  */
 
 #include "config.h"
@@ -217,22 +217,21 @@ void close_logpipes(int *logfds)
 
 void flush_logs(int logfd, FILE *std)
 {
-	while (1) {
-		struct knet_log_msg msg;
+	struct knet_log_msg msg;
+	int len;
 
-		for (size_t bytes_read = 0; bytes_read < sizeof(msg); ) {
-			int len = read(logfd, &msg + bytes_read,
-				       sizeof(msg) - bytes_read);
-			if (len <= 0) {
-				/*
-				 * clear errno to avoid incorrect propagation
-				 */
-				errno = 0;
-				return;
-			}
-			bytes_read += len;
+	while (1) {
+		len = read(logfd, &msg, sizeof(msg));
+		if (len != sizeof(msg)) {
+			/*
+			 * clear errno to avoid incorrect propagation
+			 */
+			errno = 0;
+			return;
 		}
 
+		msg.msg[sizeof(msg.msg) - 1] = 0;
+
 		fprintf(std, "[knet]: [%s] %s: %.*s\n",
 			knet_log_get_loglevel_name(msg.msglevel),
 			knet_log_get_subsystem_name(msg.subsystem),
@@ -395,6 +394,11 @@ int knet_handle_stop(knet_handle_t knet_h)
 		return -1;
 	}
 
+	if (knet_handle_setfwd(knet_h, 0) < 0) {
+		printf("knet_handle_setfwd failed: %s\n", strerror(errno));
+		return -1;
+	}
+
 	if (knet_host_get_host_list(knet_h, host_ids, &host_ids_entries) < 0) {
 		printf("knet_host_get_host_list failed: %s\n", strerror(errno));
 		return -1;
@@ -464,6 +468,15 @@ int make_local_sockaddr6(struct sockaddr_storage *lo, uint16_t offset)
 	return _make_local_sockaddr(lo, offset, AF_INET6);
 }
 
+void test_sleep(knet_handle_t knet_h, int seconds)
+{
+	if (is_memcheck() || is_helgrind()) {
+		printf("Test suite is running under valgrind, adjusting sleep timers\n");
+		seconds = seconds * 16;
+	}
+	sleep(seconds);
+}
+
 int wait_for_host(knet_handle_t knet_h, uint16_t host_id, int seconds, int logfd, FILE *std)
 {
 	int i = 0;
@@ -476,6 +489,8 @@ int wait_for_host(knet_handle_t knet_h, uint16_t host_id, int seconds, int logfd
 	while (i < seconds) {
 		flush_logs(logfd, std);
 		if (knet_h->host_index[host_id]->status.reachable == 1) {
+			printf("Waiting for host to settle\n");
+			test_sleep(knet_h, 1);
 			return 0;
 		}
 		printf("waiting host %u to be reachable for %d more seconds\n", host_id, seconds - i);
@@ -485,7 +500,7 @@ int wait_for_host(knet_handle_t knet_h, uint16_t host_id, int seconds, int logfd
 	return -1;
 }
 
-int wait_for_packet(knet_handle_t knet_h, int seconds, int datafd)
+int wait_for_packet(knet_handle_t knet_h, int seconds, int datafd, int logfd, FILE *std)
 {
 	fd_set rfds;
 	struct timeval tv;
@@ -500,7 +515,7 @@ try_again:
 	FD_ZERO(&rfds);
 	FD_SET(datafd, &rfds);
 
-	tv.tv_sec = seconds;
+	tv.tv_sec = 1;
 	tv.tv_usec = 0;
 
 	err = select(datafd+1, &rfds, NULL, NULL, &tv);
@@ -509,7 +524,8 @@ try_again:
 	 * pick an arbitrary 10 times loop (multiplied by waiting seconds)
 	 * before failing.
 	 */
-	if ((!err) && (i < 10)) {
+	if ((!err) && (i < seconds)) {
+		flush_logs(logfd, std);
 		i++;
 		goto try_again;
 	}
