@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2012-2020 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
@@ -128,9 +128,6 @@ static int pckt_defrag(knet_handle_t knet_h, struct knet_header *inbuf, ssize_t
 
 	defrag_buf_idx = find_pckt_defrag_buf(knet_h, inbuf);
 	if (defrag_buf_idx < 0) {
-		if (errno == ETIME) {
-			log_debug(knet_h, KNET_SUB_RX, "Defrag buffer expired");
-		}
 		return 1;
 	}
 
@@ -339,14 +336,9 @@ static void _parse_recv_from_links(knet_handle_t knet_h, int sockfd, const struc
 	switch (inbuf->kh_type) {
 	case KNET_HEADER_TYPE_HOST_INFO:
 	case KNET_HEADER_TYPE_DATA:
-		/*
-		 * TODO: should we accept data even if we can't reply to the other node?
-		 *       how would that work with SCTP and guaranteed delivery?
-		 */
-
 		if (!src_host->status.reachable) {
-			log_debug(knet_h, KNET_SUB_RX, "Source host %u not reachable yet", src_host->host_id);
-			//return;
+			log_debug(knet_h, KNET_SUB_RX, "Source host %u not reachable yet. Discarding packet.", src_host->host_id);
+			return;
 		}
 		inbuf->khp_data_seq_num = ntohs(inbuf->khp_data_seq_num);
 		channel = inbuf->khp_data_channel;
@@ -481,11 +473,21 @@ static void _parse_recv_from_links(knet_handle_t knet_h, int sockfd, const struc
 				return;
 			}
 
+			outlen = 0;
 			memset(iov_out, 0, sizeof(iov_out));
-			iov_out[0].iov_base = (void *) inbuf->khp_data_userdata;
-			iov_out[0].iov_len = len - KNET_HEADER_DATA_SIZE;
+
+retry:
+			iov_out[0].iov_base = (void *) inbuf->khp_data_userdata + outlen;
+			iov_out[0].iov_len = len - (outlen + KNET_HEADER_DATA_SIZE);
 
 			outlen = writev(knet_h->sockfd[channel].sockfd[knet_h->sockfd[channel].is_created], iov_out, 1);
+			if ((outlen > 0) && (outlen < (ssize_t)iov_out[0].iov_len)) {
+				log_debug(knet_h, KNET_SUB_RX,
+					  "Unable to send all data to the application in one go. Expected: %zu Sent: %zd\n",
+					  iov_out[0].iov_len, outlen);
+				goto retry;
+			}
+
 			if (outlen <= 0) {
 				knet_h->sock_notify_fn(knet_h->sock_notify_fn_private_data,
 						       knet_h->sockfd[channel].sockfd[0],
@@ -579,34 +581,36 @@ static void _parse_recv_from_links(knet_handle_t knet_h, int sockfd, const struc
 		}
 
 retry_pong:
-		if (transport_get_connection_oriented(knet_h, src_link->transport) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
-			len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL,
-				     (struct sockaddr *) &src_link->dst_addr, sizeof(struct sockaddr_storage));
-		} else {
-			len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL, NULL, 0);
-		}
-		savederrno = errno;
-		if (len != outlen) {
-			err = transport_tx_sock_error(knet_h, src_link->transport, src_link->outsock, len, savederrno);
-			switch(err) {
-				case -1: /* unrecoverable error */
-					log_debug(knet_h, KNET_SUB_RX,
-						  "Unable to send pong reply (sock: %d) packet (sendto): %d %s. recorded src ip: %s src port: %s dst ip: %s dst port: %s",
-						  src_link->outsock, errno, strerror(errno),
-						  src_link->status.src_ipaddr, src_link->status.src_port,
-						  src_link->status.dst_ipaddr, src_link->status.dst_port);
-					src_link->status.stats.tx_pong_errors++;
-					break;
-				case 0: /* ignore error and continue */
-					break;
-				case 1: /* retry to send those same data */
-					src_link->status.stats.tx_pong_retries++;
-					goto retry_pong;
-					break;
+		if (src_link->transport_connected) {
+			if (transport_get_connection_oriented(knet_h, src_link->transport) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
+				len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL,
+					     (struct sockaddr *) &src_link->dst_addr, sizeof(struct sockaddr_storage));
+			} else {
+				len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL, NULL, 0);
 			}
+			savederrno = errno;
+			if (len != outlen) {
+				err = transport_tx_sock_error(knet_h, src_link->transport, src_link->outsock, len, savederrno);
+				switch(err) {
+					case -1: /* unrecoverable error */
+						log_debug(knet_h, KNET_SUB_RX,
+							  "Unable to send pong reply (sock: %d) packet (sendto): %d %s. recorded src ip: %s src port: %s dst ip: %s dst port: %s",
+							  src_link->outsock, errno, strerror(errno),
+							  src_link->status.src_ipaddr, src_link->status.src_port,
+							  src_link->status.dst_ipaddr, src_link->status.dst_port);
+						src_link->status.stats.tx_pong_errors++;
+						break;
+					case 0: /* ignore error and continue */
+						break;
+					case 1: /* retry to send those same data */
+						src_link->status.stats.tx_pong_retries++;
+						goto retry_pong;
+						break;
+				}
+			}
+			src_link->status.stats.tx_pong_packets++;
+			src_link->status.stats.tx_pong_bytes += outlen;
 		}
-		src_link->status.stats.tx_pong_packets++;
-		src_link->status.stats.tx_pong_bytes += outlen;
 		break;
 	case KNET_HEADER_TYPE_PONG:
 		src_link->status.stats.rx_pong_packets++;
@@ -622,11 +626,21 @@ retry_pong:
 				  "Incoming pong packet from host: %u link: %u has higher latency than pong_timeout. Discarding",
 				  src_host->host_id, src_link->link_id);
 		} else {
+
+			/*
+			 * in words : ('previous mean' * '(count -1)') + 'new value') / 'count'
+			 */
+
+			src_link->latency_cur_samples++;
+
+			/*
+			 * limit to max_samples (precision)
+			 */
+			if (src_link->latency_cur_samples >= src_link->latency_max_samples) {
+				src_link->latency_cur_samples = src_link->latency_max_samples;
+			}
 			src_link->status.latency =
-				((src_link->status.latency * src_link->latency_exp) +
-				((latency_last / 1000llu) *
-					(src_link->latency_fix - src_link->latency_exp))) /
-						src_link->latency_fix;
+				(((src_link->status.latency * (src_link->latency_cur_samples - 1)) + (latency_last / 1000llu)) / src_link->latency_cur_samples);
 
 			if (src_link->status.latency < src_link->pong_timeout_adj) {
 				if (!src_link->status.connected) {
@@ -648,10 +662,15 @@ retry_pong:
 			if (src_link->status.latency < src_link->status.stats.latency_min) {
 				src_link->status.stats.latency_min = src_link->status.latency;
 			}
-			src_link->status.stats.latency_ave =
-				(src_link->status.stats.latency_ave * src_link->status.stats.latency_samples +
-				 src_link->status.latency) / (src_link->status.stats.latency_samples+1);
-			src_link->status.stats.latency_samples++;
+
+			/*
+			 * those 2 lines below make all latency average calculations consistent and capped to
+			 * link precision. In future we will kill the one above to keep only this one in
+			 * the stats structure, but for now we leave it around to avoid API/ABI
+			 * breakage as we backport the fixes to stable
+			 */
+			src_link->status.stats.latency_ave = src_link->status.latency;
+			src_link->status.stats.latency_samples = src_link->latency_cur_samples;
 		}
 		break;
 	case KNET_HEADER_TYPE_PMTUD:
@@ -680,32 +699,34 @@ retry_pong:
 			goto out_pmtud;
 		}
 retry_pmtud:
-		if (transport_get_connection_oriented(knet_h, src_link->transport) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
-			len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL,
-				     (struct sockaddr *) &src_link->dst_addr, sizeof(struct sockaddr_storage));
-		} else {
-			len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL, NULL, 0);
-		}
-		savederrno = errno;
-		if (len != outlen) {
-			err = transport_tx_sock_error(knet_h, src_link->transport, src_link->outsock, len, savederrno);
-			switch(err) {
-				case -1: /* unrecoverable error */
-					log_debug(knet_h, KNET_SUB_RX,
-						  "Unable to send PMTUd reply (sock: %d) packet (sendto): %d %s. recorded src ip: %s src port: %s dst ip: %s dst port: %s",
-						  src_link->outsock, errno, strerror(errno),
-						  src_link->status.src_ipaddr, src_link->status.src_port,
-						  src_link->status.dst_ipaddr, src_link->status.dst_port);
+		if (src_link->transport_connected) {
+			if (transport_get_connection_oriented(knet_h, src_link->transport) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
+				len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL,
+					     (struct sockaddr *) &src_link->dst_addr, sizeof(struct sockaddr_storage));
+			} else {
+				len = sendto(src_link->outsock, outbuf, outlen, MSG_DONTWAIT | MSG_NOSIGNAL, NULL, 0);
+			}
+			savederrno = errno;
+			if (len != outlen) {
+				err = transport_tx_sock_error(knet_h, src_link->transport, src_link->outsock, len, savederrno);
+				switch(err) {
+					case -1: /* unrecoverable error */
+						log_debug(knet_h, KNET_SUB_RX,
+							  "Unable to send PMTUd reply (sock: %d) packet (sendto): %d %s. recorded src ip: %s src port: %s dst ip: %s dst port: %s",
+							  src_link->outsock, errno, strerror(errno),
+							  src_link->status.src_ipaddr, src_link->status.src_port,
+							  src_link->status.dst_ipaddr, src_link->status.dst_port);
 
-					src_link->status.stats.tx_pmtu_errors++;
-					break;
-				case 0: /* ignore error and continue */
-					src_link->status.stats.tx_pmtu_errors++;
-					break;
-				case 1: /* retry to send those same data */
-					src_link->status.stats.tx_pmtu_retries++;
-					goto retry_pmtud;
-					break;
+						src_link->status.stats.tx_pmtu_errors++;
+						break;
+					case 0: /* ignore error and continue */
+						src_link->status.stats.tx_pmtu_errors++;
+						break;
+					case 1: /* retry to send those same data */
+						src_link->status.stats.tx_pmtu_retries++;
+						goto retry_pmtud;
+						break;
+				}
 			}
 		}
 		pthread_mutex_unlock(&knet_h->tx_mutex);
@@ -797,18 +818,18 @@ static void _handle_recv_from_links(knet_handle_t knet_h, int sockfd, struct kne
 		 */
 
 		switch(err) {
-			case -1: /* on error */
+			case KNET_TRANSPORT_RX_ERROR: /* on error */
 				log_debug(knet_h, KNET_SUB_RX, "Transport reported error parsing packet");
 				goto exit_unlock;
 				break;
-			case 0: /* packet is not data and we should continue the packet process loop */
+			case KNET_TRANSPORT_RX_NOT_DATA_CONTINUE: /* packet is not data and we should continue the packet process loop */
 				log_debug(knet_h, KNET_SUB_RX, "Transport reported no data, continue");
 				break;
-			case 1: /* packet is not data and we should STOP the packet process loop */
+			case KNET_TRANSPORT_RX_NOT_DATA_STOP: /* packet is not data and we should STOP the packet process loop */
 				log_debug(knet_h, KNET_SUB_RX, "Transport reported no data, stop");
 				goto exit_unlock;
 				break;
-			case 2: /* packet is data and should be parsed as such */
+			case KNET_TRANSPORT_RX_IS_DATA: /* packet is data and should be parsed as such */
 				/*
 				 * processing incoming packets vs access lists
 				 */
@@ -836,6 +857,13 @@ static void _handle_recv_from_links(knet_handle_t knet_h, int sockfd, struct kne
 				}
 				_parse_recv_from_links(knet_h, sockfd, &msg[i]);
 				break;
+			case KNET_TRANSPORT_RX_OOB_DATA_CONTINUE:
+				log_debug(knet_h, KNET_SUB_RX, "Transport is processing sock OOB data, continue");
+				break;
+			case KNET_TRANSPORT_RX_OOB_DATA_STOP:
+				log_debug(knet_h, KNET_SUB_RX, "Transport has completed processing sock OOB data, stop");
+				goto exit_unlock;
+				break;
 		}
 	}
 
