@@ -1,9 +1,9 @@
 /*
- * Copyright (C) 2016-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2020 Red Hat, Inc.  All rights reserved.
  *
  * Author: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #include "config.h"
@@ -41,6 +41,12 @@ int _recvmmsg(int sockfd, struct knet_mmsghdr *msgvec, unsigned int vlen, unsign
 		savederrno = errno;
 		if (err >= 0) {
 			msgvec[i].msg_len = err;
+			if (err == 0) {
+				/* No point in reading anything more until we know this has been dealt with
+				   or we'll just get a vector full of them. Several in fact */
+				i++;
+				break;
+			}
 		} else {
 			if ((i > 0) &&
 			    ((errno == EAGAIN) || (errno == EWOULDBLOCK))) {
@@ -58,13 +64,19 @@ int _sendmmsg(int sockfd, int connection_oriented, struct knet_mmsghdr *msgvec,
 {
 	int savederrno = 0, err = 0;
 	unsigned int i;
+	struct msghdr temp_msg;
+	struct msghdr *use_msghdr;
 
 	for (i = 0; i < vlen; i++) {
 		if (connection_oriented == TRANSPORT_PROTO_IS_CONNECTION_ORIENTED) {
-			msgvec[i].msg_hdr.msg_name = NULL;
-			msgvec[i].msg_hdr.msg_namelen = 0;
+			memcpy(&temp_msg, &msgvec[i].msg_hdr, sizeof(struct msghdr));
+			temp_msg.msg_name = NULL;
+			temp_msg.msg_namelen = 0;
+			use_msghdr = &temp_msg;
+		} else {
+			use_msghdr = &msgvec[i].msg_hdr;
 		}
-		err = sendmsg(sockfd, &msgvec[i].msg_hdr, flags);
+		err = sendmsg(sockfd, use_msghdr, flags);
 		savederrno = errno;
 		if (err < 0) {
 			break;
@@ -405,7 +417,7 @@ int _is_valid_fd(knet_handle_t knet_h, int sockfd)
 		return -1;
 	}
 
-	if (sockfd > KNET_MAX_FDS) {
+	if (sockfd >= KNET_MAX_FDS) {
 		errno = EINVAL;
 		return -1;
 	}
@@ -430,7 +442,7 @@ int _set_fd_tracker(knet_handle_t knet_h, int sockfd, uint8_t transport, uint8_t
 		return -1;
 	}
 
-	if (sockfd > KNET_MAX_FDS) {
+	if (sockfd >= KNET_MAX_FDS) {
 		errno = EINVAL;
 		return -1;
 	}
