@@ -1,9 +1,9 @@
 /*
- * Copyright (C) 2016-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2020 Red Hat, Inc.  All rights reserved.
  *
  * Author: Christine Caulfield <ccaulfie@redhat.com>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #include "config.h"
@@ -15,13 +15,17 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <stdlib.h>
+#include <assert.h>
 
 #include "compat.h"
 #include "host.h"
 #include "links.h"
+#include "links_acl.h"
+#include "links_acl_ip.h"
 #include "logging.h"
 #include "common.h"
 #include "transport_common.h"
+#include "transports.h"
 #include "threads_common.h"
 
 #ifdef HAVE_NETINET_SCTP_H
@@ -61,6 +65,7 @@ typedef struct sctp_listen_link_info {
 	struct sockaddr_storage src_address;
 	int on_listener_epoll;
 	int on_rx_epoll;
+	int sock_shutdown;
 } sctp_listen_link_info_t;
 
 typedef struct sctp_accepted_link_info {
@@ -75,9 +80,9 @@ typedef struct sctp_connect_link_info {
 	struct knet_link *link;
 	struct sockaddr_storage dst_address;
 	int connect_sock;
-	int on_connected_epoll;
 	int on_rx_epoll;
 	int close_sock;
+	int sock_shutdown;
 } sctp_connect_link_info_t;
 
 /*
@@ -94,37 +99,36 @@ typedef struct sctp_connect_link_info {
 static int _close_connect_socket(knet_handle_t knet_h, struct knet_link *kn_link)
 {
 	int err = 0, savederrno = 0;
-	sctp_connect_link_info_t *info = kn_link->transport_link;
-	sctp_handle_info_t *handle_info = knet_h->transports[KNET_TRANSPORT_SCTP];
 	struct epoll_event ev;
+	sctp_connect_link_info_t *info = kn_link->transport_link;
 
-	if (info->on_connected_epoll) {
-		memset(&ev, 0, sizeof(struct epoll_event));
-		ev.events = EPOLLOUT;
-		ev.data.fd = info->connect_sock;
-		if (epoll_ctl(handle_info->connect_epollfd, EPOLL_CTL_DEL, info->connect_sock, &ev)) {
-			savederrno = errno;
-			err = -1;
-			log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to remove connected socket from the epoll pool: %s",
-				strerror(errno));
-			goto exit_error;
-		}
-		info->on_connected_epoll = 0;
-	}
-
-exit_error:
 	if (info->connect_sock != -1) {
+		if (info->on_rx_epoll) {
+			memset(&ev, 0, sizeof(struct epoll_event));
+			ev.events = EPOLLIN;
+			ev.data.fd = info->connect_sock;
+			if (epoll_ctl(knet_h->recv_from_links_epollfd, EPOLL_CTL_DEL, info->connect_sock, &ev)) {
+				savederrno = errno;
+				err = -1;
+				log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to remove connected socket from epoll pool: %s",
+				strerror(savederrno));
+				goto exit_error;
+			}
+			info->on_rx_epoll = 0;
+		}
+
 		if (_set_fd_tracker(knet_h, info->connect_sock, KNET_MAX_TRANSPORTS, SCTP_NO_LINK_INFO, NULL) < 0) {
 			savederrno = errno;
 			err = -1;
 			log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to set fd tracker: %s",
 				strerror(savederrno));
-			goto exit_error;
+		} else {
+			close(info->connect_sock);
+			info->connect_sock = -1;
 		}
-		close(info->connect_sock);
-		info->connect_sock = -1;
 	}
 
+exit_error:
 	errno = savederrno;
 	return err;
 }
@@ -188,34 +192,17 @@ static int _reconnect_socket(knet_handle_t knet_h, struct knet_link *kn_link)
 {
 	int err = 0, savederrno = 0;
 	sctp_connect_link_info_t *info = kn_link->transport_link;
-	sctp_handle_info_t *handle_info = knet_h->transports[KNET_TRANSPORT_SCTP];
-	struct epoll_event ev;
 
 	if (connect(info->connect_sock, (struct sockaddr *)&kn_link->dst_addr, sockaddr_len(&kn_link->dst_addr)) < 0) {
-		if ((errno != EALREADY) && (errno != EINPROGRESS) && (errno != EISCONN)) {
-			savederrno = errno;
+		savederrno = errno;
+		log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "SCTP socket %d received error: %s", info->connect_sock, strerror(savederrno));
+		if ((savederrno != EALREADY) && (savederrno != EINPROGRESS) && (savederrno != EISCONN)) {
 			err = -1;
 			log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to connect SCTP socket %d: %s",
 				info->connect_sock, strerror(savederrno));
-			goto exit_error;
 		}
 	}
 
-	if (!info->on_connected_epoll) {
-		memset(&ev, 0, sizeof(struct epoll_event));
-		ev.events = EPOLLOUT;
-		ev.data.fd = info->connect_sock;
-		if (epoll_ctl(handle_info->connect_epollfd, EPOLL_CTL_ADD, info->connect_sock, &ev)) {
-			savederrno = errno;
-			err = -1;
-			log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to add send/recv to epoll pool: %s",
-				strerror(savederrno));
-			goto exit_error;
-		}
-		info->on_connected_epoll = 1;
-	}
-
-exit_error:
 	errno = savederrno;
 	return err;
 }
@@ -223,9 +210,8 @@ exit_error:
 static int _create_connect_socket(knet_handle_t knet_h, struct knet_link *kn_link)
 {
 	int err = 0, savederrno = 0;
-	sctp_connect_link_info_t *info = kn_link->transport_link;
-	sctp_handle_info_t *handle_info = knet_h->transports[KNET_TRANSPORT_SCTP];
 	struct epoll_event ev;
+	sctp_connect_link_info_t *info = kn_link->transport_link;
 	int connect_sock;
 
 	connect_sock = socket(kn_link->dst_addr.ss_family, SOCK_STREAM, IPPROTO_SCTP);
@@ -251,8 +237,19 @@ static int _create_connect_socket(knet_handle_t knet_h, struct knet_link *kn_lin
 		goto exit_error;
 	}
 
+	memset(&ev, 0, sizeof(struct epoll_event));
+	ev.events = EPOLLIN;
+	ev.data.fd = connect_sock;
+	if (epoll_ctl(knet_h->recv_from_links_epollfd, EPOLL_CTL_ADD, connect_sock, &ev)) {
+		log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to add connected socket to epoll pool: %s",
+			strerror(errno));
+	}
+	info->on_rx_epoll = 1;
+
 	info->connect_sock = connect_sock;
 	info->close_sock = 0;
+	kn_link->outsock = info->connect_sock;
+
 	if (_reconnect_socket(knet_h, kn_link) < 0) {
 		savederrno = errno;
 		err = -1;
@@ -261,9 +258,6 @@ static int _create_connect_socket(knet_handle_t knet_h, struct knet_link *kn_lin
 
 exit_error:
 	if (err) {
-		if (info->on_connected_epoll) {
-			epoll_ctl(handle_info->connect_epollfd, EPOLL_CTL_DEL, connect_sock, &ev);
-		}
 		if (connect_sock >= 0) {
 			close(connect_sock);
 		}
@@ -272,6 +266,32 @@ exit_error:
 	return err;
 }
 
+static void _lock_sleep_relock(knet_handle_t knet_h)
+{
+	int i = 0;
+
+	/* Don't hold onto the lock while sleeping */
+	pthread_rwlock_unlock(&knet_h->global_rwlock);
+
+	while (i < 5) {
+		usleep(KNET_THREADS_TIMERES / 16);
+		if (!pthread_rwlock_rdlock(&knet_h->global_rwlock)) {
+			/*
+			 * lock acquired, we can go out
+			 */
+			return;
+		} else {
+			log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to get read lock!");
+			i++;
+		}
+	}
+	/*
+	 * time to crash! if we cannot re-acquire the lock
+	 * there is no easy way out of this one
+	 */
+	assert(0);
+}
+
 int sctp_transport_tx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err, int recv_errno)
 {
 	sctp_connect_link_info_t *connect_info = knet_h->knet_transport_fd_tracker[sockfd].data;
@@ -298,10 +318,7 @@ int sctp_transport_tx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err,
 #ifdef DEBUG
 			log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Sock: %d is overloaded. Slowing TX down", sockfd);
 #endif
-			/* Don't hold onto the lock while sleeping */
-			pthread_rwlock_unlock(&knet_h->global_rwlock);
-			usleep(KNET_THREADS_TIMERES / 16);
-			pthread_rwlock_rdlock(&knet_h->global_rwlock);
+			_lock_sleep_relock(knet_h);
 			return 1;
 		}
 		return -1;
@@ -319,22 +336,21 @@ int sctp_transport_tx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err,
  *       because scheduling can make it so that this thread will overload
  *       and the threads supposed to take care of the error will never
  *       be able to take action.
- *       we CANNOT handle FDs here diretly (close/reconnect/etc) due
+ *       we CANNOT handle FDs here directly (close/reconnect/etc) due
  *       to locking context. We need to delegate that to their respective
- *       management threads within global write lock.
+ *       management threads within the global write lock.
  *
  * this function is called from:
  * - RX thread with recv_err <= 0 directly on recvmmsg error
  * - transport_rx_is_data when msg_len == 0 (recv_err = 1)
  * - transport_rx_is_data on notification (recv_err = 2)
  *
- * basically this small abouse of recv_err is to detect notifications
+ * basically this small abuse of recv_err is to detect notifications
  * generated by sockets created by listen().
  */
 int sctp_transport_rx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err, int recv_errno)
 {
 	struct epoll_event ev;
-	sctp_connect_link_info_t *connect_info = knet_h->knet_transport_fd_tracker[sockfd].data;
 	sctp_accepted_link_info_t *accepted_info = knet_h->knet_transport_fd_tracker[sockfd].data;
 	sctp_listen_link_info_t *listen_info;
 	sctp_handle_info_t *handle_info = knet_h->transports[KNET_TRANSPORT_SCTP];
@@ -350,18 +366,6 @@ int sctp_transport_rx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err,
 			 * they follow a notification (double notification)
 			 */
 			if (recv_err != 1) {
-				connect_info->link->transport_connected = 0;
-				if (connect_info->on_rx_epoll) {
-					memset(&ev, 0, sizeof(struct epoll_event));
-					ev.events = EPOLLIN;
-					ev.data.fd = sockfd;
-					if (epoll_ctl(knet_h->recv_from_links_epollfd, EPOLL_CTL_DEL, sockfd, &ev)) {
-						log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to remove EOFed socket from epoll pool: %s",
-						strerror(errno));
-						return -1;
-					}
-					connect_info->on_rx_epoll = 0;
-				}
 				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Notifying connect thread that sockfd %d received an error", sockfd);
 				if (sendto(handle_info->connectsockfd[1], &sockfd, sizeof(int), MSG_DONTWAIT | MSG_NOSIGNAL, NULL, 0) != sizeof(int)) {
 					log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to notify connect thread: %s", strerror(errno));
@@ -404,10 +408,7 @@ int sctp_transport_rx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err,
 	 * Under RX pressure we need to give time to IPC to pick up the message
 	 */
 
-	/* Don't hold onto the lock while sleeping */
-	pthread_rwlock_unlock(&knet_h->global_rwlock);
-	usleep(KNET_THREADS_TIMERES / 2);
-	pthread_rwlock_rdlock(&knet_h->global_rwlock);
+	_lock_sleep_relock(knet_h);
 	return 0;
 }
 
@@ -423,24 +424,34 @@ int sctp_transport_rx_is_data(knet_handle_t knet_h, int sockfd, struct knet_mmsg
 	size_t iovlen = msg->msg_hdr.msg_iovlen;
 	struct sctp_assoc_change *sac;
 	union sctp_notification  *snp;
-	sctp_accepted_link_info_t *info = knet_h->knet_transport_fd_tracker[sockfd].data;
+	sctp_accepted_link_info_t *listen_info = knet_h->knet_transport_fd_tracker[sockfd].data;
+	sctp_connect_link_info_t *connect_info = knet_h->knet_transport_fd_tracker[sockfd].data;
 
 	if (!(msg->msg_hdr.msg_flags & MSG_NOTIFICATION)) {
 		if (msg->msg_len == 0) {
-			log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "received 0 bytes len packet: %d", sockfd);
 			/*
 			 * NOTE: with event notification enabled, we receive error twice:
 			 *       1) from the event notification
 			 *       2) followed by a 0 byte msg_len
 			 *
-			 * This is generally not a problem if not for causing extra
-			 * handling for the same issue. Should we drop notifications
-			 * and keep the code generic (handle all errors via msg_len = 0)
-			 * or keep the duplication as safety measure, or drop msg_len = 0
-			 * handling (what about sockets without events enabled?)
+			 * the event handler should take care to avoid #2 by stopping
+			 * the rx thread from processing more packets than necessary.
 			 */
-			sctp_transport_rx_sock_error(knet_h, sockfd, 1, 0);
-			return 1;
+			if (knet_h->knet_transport_fd_tracker[sockfd].data_type == SCTP_CONNECT_LINK_INFO) {
+				if (connect_info->sock_shutdown) {
+					return KNET_TRANSPORT_RX_OOB_DATA_CONTINUE;
+				}
+			} else {
+				if (listen_info->link_info->sock_shutdown) {
+					return KNET_TRANSPORT_RX_OOB_DATA_CONTINUE;
+				}
+			}
+			/*
+			 * this is pretty much dead code and we should never hit it.
+			 * keep it for safety and avoid the rx thread to process
+			 * bad info / data.
+			 */
+			return KNET_TRANSPORT_RX_NOT_DATA_STOP;
 		}
 		/*
 		 * missing MSG_EOR has to be treated as a short read
@@ -452,9 +463,9 @@ int sctp_transport_rx_is_data(knet_handle_t knet_h, int sockfd, struct knet_mmsg
 			 * copy the incoming data into mread_buf + mread_len (incremental)
 			 * and increase mread_len
 			 */
-			memmove(info->mread_buf + info->mread_len, iov->iov_base, msg->msg_len);
-			info->mread_len = info->mread_len + msg->msg_len;
-			return 0;
+			memmove(listen_info->mread_buf + listen_info->mread_len, iov->iov_base, msg->msg_len);
+			listen_info->mread_len = listen_info->mread_len + msg->msg_len;
+			return KNET_TRANSPORT_RX_NOT_DATA_CONTINUE;
 		}
 		/*
 		 * got EOR.
@@ -462,57 +473,88 @@ int sctp_transport_rx_is_data(knet_handle_t knet_h, int sockfd, struct knet_mmsg
 		 * complete reassembling the packet in mread_buf, copy it back in the iov
 		 * and set the iov/msg len numbers (size) correctly
 		 */
-		if (info->mread_len) {
+		if (listen_info->mread_len) {
 			/*
 			 * add last fragment to mread_buf
 			 */
-			memmove(info->mread_buf + info->mread_len, iov->iov_base, msg->msg_len);
-			info->mread_len = info->mread_len + msg->msg_len;
+			memmove(listen_info->mread_buf + listen_info->mread_len, iov->iov_base, msg->msg_len);
+			listen_info->mread_len = listen_info->mread_len + msg->msg_len;
 			/*
 			 * move all back into the iovec
 			 */
-			memmove(iov->iov_base, info->mread_buf, info->mread_len);
-			msg->msg_len = info->mread_len;
-			info->mread_len = 0;
+			memmove(iov->iov_base, listen_info->mread_buf, listen_info->mread_len);
+			msg->msg_len = listen_info->mread_len;
+			listen_info->mread_len = 0;
 		}
-		return 2;
+		return KNET_TRANSPORT_RX_IS_DATA;
 	}
 
 	if (!(msg->msg_hdr.msg_flags & MSG_EOR)) {
-		return 1;
+		return KNET_TRANSPORT_RX_NOT_DATA_STOP;
 	}
 
-	for (i=0; i< iovlen; i++) {
+	for (i = 0; i < iovlen; i++) {
 		snp = iov[i].iov_base;
 
 		switch (snp->sn_header.sn_type) {
 			case SCTP_ASSOC_CHANGE:
-				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp assoc change");
 				sac = &snp->sn_assoc_change;
-				if (sac->sac_state == SCTP_COMM_LOST) {
-					log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp assoc change: comm_lost");
-					sctp_transport_rx_sock_error(knet_h, sockfd, 2, 0);
+				switch (sac->sac_state) {
+					case SCTP_COMM_LOST:
+						log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp assoc change socket %d: comm_lost", sockfd);
+						sctp_transport_rx_sock_error(knet_h, sockfd, 2, 0);
+						return KNET_TRANSPORT_RX_OOB_DATA_STOP;
+						break;
+					case SCTP_COMM_UP:
+						log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp assoc change socket %d: comm_up", sockfd);
+						if (knet_h->knet_transport_fd_tracker[sockfd].data_type == SCTP_CONNECT_LINK_INFO) {
+							connect_info->link->transport_connected = 1;
+						}
+						break;
+					case SCTP_RESTART:
+						log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp assoc change socket %d: restart", sockfd);
+						break;
+					case SCTP_SHUTDOWN_COMP:
+						log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp assoc change socket %d: shutdown comp", sockfd);
+						if (knet_h->knet_transport_fd_tracker[sockfd].data_type == SCTP_CONNECT_LINK_INFO) {
+							connect_info->close_sock = 1;
+						}
+						sctp_transport_rx_sock_error(knet_h, sockfd, 2, 0);
+						return KNET_TRANSPORT_RX_OOB_DATA_STOP;
+						break;
+					case SCTP_CANT_STR_ASSOC:
+						log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp assoc change socket %d: cant str assoc", sockfd);
+						sctp_transport_rx_sock_error(knet_h, sockfd, 2, 0);
+						break;
+					default:
+						log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp assoc change socket %d: unknown %d", sockfd, sac->sac_state);
+						break;
 				}
 				break;
 			case SCTP_SHUTDOWN_EVENT:
-				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp shutdown event");
-				sctp_transport_rx_sock_error(knet_h, sockfd, 2, 0);
+				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp shutdown event socket %d", sockfd);
+				if (knet_h->knet_transport_fd_tracker[sockfd].data_type == SCTP_CONNECT_LINK_INFO) {
+					connect_info->link->transport_connected = 0;
+					connect_info->sock_shutdown = 1;
+				} else {
+					listen_info->link_info->sock_shutdown = 1;
+				}
 				break;
 			case SCTP_SEND_FAILED:
-				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp send failed");
+				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp send failed socket: %d", sockfd);
 				break;
 			case SCTP_PEER_ADDR_CHANGE:
-				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp peer addr change");
+				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp peer addr change socket %d", sockfd);
 				break;
 			case SCTP_REMOTE_ERROR:
-				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp remote error");
+				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] sctp remote error socket %d", sockfd);
 				break;
 			default:
-				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] unknown sctp event type: %hu\n", snp->sn_header.sn_type);
+				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "[event] unknown sctp event socket: %d type: %hu", sockfd, snp->sn_header.sn_type);
 				break;
 		}
 	}
-	return 0;
+	return KNET_TRANSPORT_RX_OOB_DATA_CONTINUE;
 }
 
 /*
@@ -523,25 +565,16 @@ int sctp_transport_rx_is_data(knet_handle_t knet_h, int sockfd, struct knet_mmsg
  * _handle_connected_sctp* are called with a global write lock
  * from the connect_thread
  */
-static void _handle_connected_sctp(knet_handle_t knet_h, int connect_sock)
+static void _handle_connected_sctp_socket(knet_handle_t knet_h, int connect_sock)
 {
 	int err;
-	struct epoll_event ev;
 	unsigned int status, len = sizeof(status);
-	sctp_handle_info_t *handle_info = knet_h->transports[KNET_TRANSPORT_SCTP];
 	sctp_connect_link_info_t *info = knet_h->knet_transport_fd_tracker[connect_sock].data;
 	struct knet_link *kn_link = info->link;
 
-	err = getsockopt(connect_sock, SOL_SOCKET, SO_ERROR, &status, &len);
-	if (err) {
-		log_err(knet_h, KNET_SUB_TRANSP_SCTP, "SCTP getsockopt() on connecting socket %d failed: %s",
-			connect_sock, strerror(errno));
-		return;
-	}
-
 	if (info->close_sock) {
 		if (_close_connect_socket(knet_h, kn_link) < 0) {
-			log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to close sock %d from _handle_connected_sctp: %s", connect_sock, strerror(errno));
+			log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to close sock %d from _handle_connected_sctp_socket: %s", connect_sock, strerror(errno));
 			return;
 		}
 		info->close_sock = 0;
@@ -551,6 +584,15 @@ static void _handle_connected_sctp(knet_handle_t knet_h, int connect_sock)
 		}
 	}
 
+	_reconnect_socket(knet_h, info->link);
+
+	err = getsockopt(connect_sock, SOL_SOCKET, SO_ERROR, &status, &len);
+	if (err) {
+		log_err(knet_h, KNET_SUB_TRANSP_SCTP, "SCTP getsockopt() on connecting socket %d failed: %s",
+			connect_sock, strerror(errno));
+		return;
+	}
+
 	if (status) {
 		log_info(knet_h, KNET_SUB_TRANSP_SCTP, "SCTP connect on %d to %s port %s failed: %s",
 			 connect_sock, kn_link->status.dst_ipaddr, kn_link->status.dst_port,
@@ -560,44 +602,18 @@ static void _handle_connected_sctp(knet_handle_t knet_h, int connect_sock)
 		 * No need to create a new socket if connect failed,
 		 * just retry connect
 		 */
-		_reconnect_socket(knet_h, info->link);
 		return;
 	}
 
-	/*
-	 * Connected - Remove us from the connect epoll
-	 */
-	memset(&ev, 0, sizeof(struct epoll_event));
-	ev.events = EPOLLOUT;
-	ev.data.fd = connect_sock;
-	if (epoll_ctl(handle_info->connect_epollfd, EPOLL_CTL_DEL, connect_sock, &ev)) {
-		log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to remove connected socket %d from epoll pool: %s",
-			connect_sock, strerror(errno));
-	}
-	info->on_connected_epoll = 0;
-
-	kn_link->transport_connected = 1;
-	kn_link->outsock = info->connect_sock;
-
-	memset(&ev, 0, sizeof(struct epoll_event));
-	ev.events = EPOLLIN;
-	ev.data.fd = connect_sock;
-	if (epoll_ctl(knet_h->recv_from_links_epollfd, EPOLL_CTL_ADD, connect_sock, &ev)) {
-		log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to add connected socket to epoll pool: %s",
-			strerror(errno));
-	}
-	info->on_rx_epoll = 1;
-
 	log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "SCTP handler fd %d now connected to %s port %s",
 		  connect_sock,
 		  kn_link->status.dst_ipaddr, kn_link->status.dst_port);
 }
 
-static void _handle_connected_sctp_errors(knet_handle_t knet_h)
+static void _handle_connected_sctp_notifications(knet_handle_t knet_h)
 {
 	int sockfd = -1;
 	sctp_handle_info_t *handle_info = knet_h->transports[KNET_TRANSPORT_SCTP];
-	sctp_connect_link_info_t *info;
 
 	if (recv(handle_info->connectsockfd[0], &sockfd, sizeof(int), MSG_DONTWAIT | MSG_NOSIGNAL) != sizeof(int)) {
 		log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Short read on connectsockfd");
@@ -609,13 +625,16 @@ static void _handle_connected_sctp_errors(knet_handle_t knet_h)
 		return;
 	}
 
+	/*
+	 * revalidate sockfd
+	 */
+	if ((sockfd < 0) || (sockfd >= KNET_MAX_FDS)) {
+		return;
+	}
+
 	log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Processing connected error on socket: %d", sockfd);
 
-	info = knet_h->knet_transport_fd_tracker[sockfd].data;
-
-	info->close_sock = 1;
-	info->link->transport_connected = 0;
-	_reconnect_socket(knet_h, info->link);
+	_handle_connected_sctp_socket(knet_h, sockfd);
 }
 
 static void *_sctp_connect_thread(void *data)
@@ -665,13 +684,9 @@ static void *_sctp_connect_thread(void *data)
 		for (i = 0; i < nev; i++) {
 			if (events[i].data.fd == handle_info->connectsockfd[0]) {
 				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Received notification from rx_error for connected socket");
-				_handle_connected_sctp_errors(knet_h);
+				_handle_connected_sctp_notifications(knet_h);
 			} else {
-				if (_is_valid_fd(knet_h, events[i].data.fd) == 1) {
-					_handle_connected_sctp(knet_h, events[i].data.fd);
-				} else {
-					log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Received stray notification for dead fd %d\n", events[i].data.fd);
-				}
+				log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Received stray notification on connected sockfd %d\n", events[i].data.fd);
 			}
 		}
 		pthread_rwlock_unlock(&knet_h->global_rwlock);
@@ -728,6 +743,15 @@ static void _handle_incoming_sctp(knet_handle_t knet_h, int listen_sock)
 
 	log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Incoming: received connection from: %s port: %s",
 						addr_str, port_str);
+	if (knet_h->use_access_lists) {
+		if (!check_validate(knet_h, listen_sock, KNET_TRANSPORT_SCTP, &ss)) {
+			savederrno = EINVAL;
+			log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Connection rejected from %s/%s", addr_str, port_str);
+			close(new_fd);
+			errno = savederrno;
+			return;
+		}
+	}
 
 	/*
 	 * Keep a track of all accepted FDs
@@ -793,12 +817,14 @@ static void _handle_incoming_sctp(knet_handle_t knet_h, int listen_sock)
 
 exit_error:
 	if (err) {
-		if ((i >= 0) || (i < MAX_ACCEPTED_SOCKS)) {
+		if ((i >= 0) && (i < MAX_ACCEPTED_SOCKS)) {
 			info->accepted_socks[i] = -1;
 		}
 		_set_fd_tracker(knet_h, new_fd, KNET_MAX_TRANSPORTS, SCTP_NO_LINK_INFO, NULL);
 		free(accept_info);
-		close(new_fd);
+		if (new_fd >= 0) {
+			close(new_fd);
+		}
 	}
 	errno = savederrno;
 	return;
@@ -828,6 +854,13 @@ static void _handle_listen_sctp_errors(knet_handle_t knet_h)
 		return;
 	}
 
+	/*
+	 * revalidate sockfd
+	 */
+	if ((sockfd < 0) || (sockfd >= KNET_MAX_FDS)) {
+		return;
+	}
+
 	log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Processing listen error on socket: %d", sockfd);
 
 	accept_info = knet_h->knet_transport_fd_tracker[sockfd].data;
@@ -859,6 +892,7 @@ static void _handle_listen_sctp_errors(knet_handle_t knet_h)
 			info->accepted_socks[i] = -1;
 			free(accept_info);
 			close(sockfd);
+			break; /* Keeps covscan happy */
 		}
 	}
 }
@@ -936,6 +970,10 @@ static sctp_listen_link_info_t *sctp_link_listener_start(knet_handle_t knet_h, s
 	 */
 	knet_list_for_each_entry(info, &handle_info->listen_links_list, list) {
 		if (memcmp(&info->src_address, &kn_link->src_addr, sizeof(struct sockaddr_storage)) == 0) {
+			if ((check_add(knet_h, info->listen_sock, KNET_TRANSPORT_SCTP, -1,
+				       &kn_link->dst_addr, &kn_link->dst_addr, CHECK_TYPE_ADDRESS, CHECK_ACCEPT) < 0) && (errno != EEXIST)) {
+				return NULL;
+			}
 			return info;
 		}
 	}
@@ -990,6 +1028,15 @@ static sctp_listen_link_info_t *sctp_link_listener_start(knet_handle_t knet_h, s
 		goto exit_error;
 	}
 
+	if ((check_add(knet_h, listen_sock, KNET_TRANSPORT_SCTP, -1,
+		       &kn_link->dst_addr, &kn_link->dst_addr, CHECK_TYPE_ADDRESS, CHECK_ACCEPT) < 0) && (errno != EEXIST)) {
+		savederrno = errno;
+		err = -1;
+		log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to configure default access lists: %s",
+			strerror(savederrno));
+		goto exit_error;
+	}
+
 	memset(&ev, 0, sizeof(struct epoll_event));
 	ev.events = EPOLLIN;
 	ev.data.fd = listen_sock;
@@ -1009,10 +1056,11 @@ static sctp_listen_link_info_t *sctp_link_listener_start(knet_handle_t knet_h, s
 
 exit_error:
 	if (err) {
-		if (info->on_listener_epoll) {
+		if ((info) && (info->on_listener_epoll)) {
 			epoll_ctl(handle_info->listen_epollfd, EPOLL_CTL_DEL, listen_sock, &ev);
 		}
 		if (listen_sock >= 0) {
+			check_rmall(knet_h, listen_sock, KNET_TRANSPORT_SCTP);
 			close(listen_sock);
 		}
 		if (info) {
@@ -1050,6 +1098,11 @@ static int sctp_link_listener_stop(knet_handle_t knet_h, struct knet_link *kn_li
 		}
 	}
 
+	if ((check_rm(knet_h, info->listen_sock, KNET_TRANSPORT_SCTP,
+		      &kn_link->dst_addr, &kn_link->dst_addr, CHECK_TYPE_ADDRESS, CHECK_ACCEPT) < 0) && (errno != ENOENT)) {
+		log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to remove default access lists for %d", info->listen_sock);
+	}
+
 	if (found) {
 		this_link_info->listener = NULL;
 		log_debug(knet_h, KNET_SUB_TRANSP_SCTP, "SCTP listener socket %d still in use", info->listen_sock);
@@ -1080,6 +1133,8 @@ static int sctp_link_listener_stop(knet_handle_t knet_h, struct knet_link *kn_li
 		goto exit_error;
 	}
 
+	check_rmall(knet_h, info->listen_sock, KNET_TRANSPORT_SCTP);
+
 	close(info->listen_sock);
 
 	for (i=0; i< MAX_ACCEPTED_SOCKS; i++) {
@@ -1134,7 +1189,6 @@ int sctp_transport_link_set_config(knet_handle_t knet_h, struct knet_link *kn_li
 	info->link = kn_link;
 
 	memmove(&info->dst_address, &kn_link->dst_addr, sizeof(struct sockaddr_storage));
-	info->on_connected_epoll = 0;
 	info->connect_sock = -1;
 
 	info->listener = sctp_link_listener_start(knet_h, kn_link);
@@ -1158,7 +1212,7 @@ int sctp_transport_link_set_config(knet_handle_t knet_h, struct knet_link *kn_li
 exit_error:
 	if (err) {
 		if (info) {
-			if (info->connect_sock) {
+			if (info->connect_sock >= 0) {
 				close(info->connect_sock);
 			}
 			if (info->listener) {
@@ -1179,7 +1233,6 @@ int sctp_transport_link_clear_config(knet_handle_t knet_h, struct knet_link *kn_
 {
 	int err = 0, savederrno = 0;
 	sctp_connect_link_info_t *info;
-	struct epoll_event ev;
 
 	if (!kn_link) {
 		errno = EINVAL;
@@ -1196,25 +1249,11 @@ int sctp_transport_link_clear_config(knet_handle_t knet_h, struct knet_link *kn_
 	if ((sctp_link_listener_stop(knet_h, kn_link) <0) && (errno != EBUSY)) {
 		savederrno = errno;
 		err = -1;
-		log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to remove listener trasport: %s",
+		log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to remove listener transport: %s",
 			strerror(savederrno));
 		goto exit_error;
 	}
 
-	if (info->on_rx_epoll) {
-		memset(&ev, 0, sizeof(struct epoll_event));
-		ev.events = EPOLLIN;
-		ev.data.fd = info->connect_sock;
-		if (epoll_ctl(knet_h->recv_from_links_epollfd, EPOLL_CTL_DEL, info->connect_sock, &ev)) {
-			savederrno = errno;
-			err = -1;
-			log_err(knet_h, KNET_SUB_TRANSP_SCTP, "Unable to remove connected socket from epoll pool: %s",
-			strerror(savederrno));
-			goto exit_error;
-		}
-		info->on_rx_epoll = 0;
-	}
-
 	if (_close_connect_socket(knet_h, kn_link) < 0) {
 		savederrno = errno;
 		err = -1;
@@ -1504,4 +1543,11 @@ int sctp_transport_link_dyn_connect(knet_handle_t knet_h, int sockfd, struct kne
 	kn_link->transport_connected = 1;
 	return 0;
 }
+
+int sctp_transport_link_get_acl_fd(knet_handle_t knet_h, struct knet_link *kn_link)
+{
+	sctp_connect_link_info_t *this_link_info = kn_link->transport_link;
+	sctp_listen_link_info_t *info = this_link_info->listener;
+	return info->listen_sock;
+}
 #endif
